import json
import os
import yaml

from argparse import ArgumentTypeError


def validateJsonData(data: str) -> dict:
    return json.loads(data)


def validateYamlData(data: str) -> dict:
    return yaml.safe_load(data)


def validateFile(file_path: str) -> dict:
    try:
        with open(file_path, 'r') as f:
            data = ''.join(f.readlines())
    except FileNotFoundError:
        raise ArgumentTypeError(f'no file: {file_path}')

    if os.path.splitext(file_path)[1] in ('.yml', '.yaml'):
        return validateYamlData(data)

    return validateJsonData(data)



import argparse
import os

from tempfile import TemporaryDirectory
from tmpl8 import __version__
from tmpl8.command import Command
from tmpl8.data import validateJsonData, validateFile
from tmpl8.template import templateFile


def main():
    parser = argparse.ArgumentParser(
        prog='tmpl8',
        description="tmpl8 wrapper for cli commands"
    )
    parser.add_argument(
        '-v', '--version',
        action='version',
        version=__version__
    )
    command_group = parser.add_argument_group('command')
    command_group.add_argument(
        'command',
        help='command to run',
        nargs=1,
        default=[]
    )
    command_group.add_argument(
        'args',
        nargs=argparse.REMAINDER,
        help=argparse.SUPPRESS
    )

    data_group = parser.add_argument_group('data', 'data used when templating')
    data_group.add_argument(
        '-d', '--data',
        help=(
            'json data for templating.'
            'Overwrites data from file if -f/--file is specified'
        ),
        type=validateJsonData
    )
    data_group.add_argument(
        '-f', '--file',
        help='file with data to use for templating',
        type=validateFile
    )

    args = parser.parse_args()

    error_message = 'error:'

    if args.command == []:
        error_message = f'{error_message} missing command'

    if not args.data and not args.file:
        error_message = f'{error_message} missing data'

    if error_message != 'error:':
        parser.print_help()
        parser.exit(status=1, message=f'{error_message}\n')

    command = Command(args.command + args.args)

    args.data = args.data or {}
    args.file = args.file or {}

    data = {**args.file, **args.data}

    with TemporaryDirectory() as td:
        for arg_info in command.arg_info:
            dir = td
            if arg_info.arg_type == 'dir':
                dir = os.path.join(td, os.path.basename(arg_info.path))
                os.mkdir(dir)
            for file in arg_info.files:
                arg_info.template_dir = dir
                arg_info.template_files.append(templateFile(file, dir, data))

        result = command.generateCommand().runCommand()

        if result.returncode == 0:
            print(result.stdout)
        else:
            print(result.stderr)
            exit(result.returncode)



from dataclasses import dataclass, field
from enum import Enum
import glob
import os
import subprocess
import sys


class ArgType(Enum):
    ARG = 'arg'
    PATH = 'path'
    IGNORE = 'ignore'
    FILE = 'file'
    DIR = 'dir'


@dataclass
class ArgInfo:
    arg: str
    arg_type: ArgType
    path: str = ''
    files: set = field(default_factory=set)
    template_files: list = field(default_factory=list)
    template_dir: str = ''


class Command:
    '''
    This Command class holds all the information needed to construct
    a new command using templated files.
    '''

    def __init__(self, command: list):
        if not command:
            raise ValueError('command must be defined.')

        self.command: list[str] = command
        self.arg_info: list[ArgInfo] = self._getCommandInfo()

    def _getCommandInfo(self) -> list:
        '''
        This method takes the command the Command was initiated with and extracts
        any potential files from the command. This method will auto-run on object init.
        '''

        arg_info = []

        for arg in self.command:

            arg_to_path = glob.glob(os.path.expandvars(arg))

            if not arg_to_path:
                arg_info.append(
                    ArgInfo(
                        arg=arg,
                        arg_type=ArgType.ARG
                    )
                )

            for path in arg_to_path:
                path = os.path.realpath(os.path.expanduser(path))

                if 'pycache' in arg:
                    arg_info.append(
                        ArgInfo(
                            arg=arg,
                            arg_type=ArgType.IGNORE
                        )
                    )
                    continue

                if os.path.isfile(path):
                    arg_info.append(
                        ArgInfo(
                            arg=arg,
                            arg_type=ArgType.FILE,
                            path=path,
                            files={os.path.abspath(path)}
                        )
                    )

                elif os.path.isdir(path):
                    dir_files = set()

                    for root, dir, files in os.walk(path):
                        if '.git' in root:
                            continue

                        for file in files:
                            file = os.path.join(root, file)
                            ignore_paths = ['pycache', '.terraform']

                            if (os.path.isfile(file) and
                                    not any(x in file for x in ignore_paths)):
                                dir_files.add(file)

                    arg_info.append(
                        ArgInfo(
                            arg=arg,
                            arg_type=ArgType.DIR,
                            path=path,
                            files=dir_files
                        )
                    )
        return arg_info

    def generateCommand(self) -> list:
        '''
        This method generates the new_command attribute based on the
        arg_info generated by _extractFiles.s
        '''
        new_command = []

        if not self.arg_info:
            raise ValueError('Object does not have arg_info defined.')

        for arg_info in self.arg_info:
            if arg_info.arg_type == ArgType.DIR:
                new_command.append(arg_info.template_dir or arg_info.arg)
            elif arg_info.arg_type == ArgType.FILE:
                new_command.append(*arg_info.template_files or arg_info.arg)
            elif arg_info.arg_type == ArgType.IGNORE:
                continue
            elif arg_info.arg_type == ArgType.ARG:
                new_command.append(arg_info.arg)

        self.new_command = new_command

        return new_command

    def runCommand(self) -> subprocess.CompletedProcess:
        '''
        This method acts as a helper function to run the inputted command.
        '''
        try:
            result = subprocess.run(
                self.new_command,
                capture_output=True,
                text=True
            )
        except KeyboardInterrupt:
            print('--KeyboardInterrupt--', file=sys.stderr)
            exit(1)

        return result



from setuptools import setup, find_packages
from tmpl8 import __version__

install_requires = [
    'pyyaml',
    'jinja2'
]

setup_options = dict(
    name='tmpl8',
    version=__version__,
    description='Templating wrapper for CLI commands',
    packages=find_packages(include=('tmpl8', 'tmpl8.*')),
    author='Matthew Altberg',
    install_requires=install_requires,
    extras_require={
        'dev': [
            'pytest',
            'flake8',
            'pre-commit'
        ]
    },
    entry_points={
        'console_scripts': [
            'tmpl8=tmpl8.cli:main'
        ]
    },
    python_requires=">=3.7",
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        'Programming Language :: Python :: 3.12'
    ]
)

setup(**setup_options)



import os
from typing import Optional

from jinja2 import Environment, FileSystemLoader


def templateFile(input_path: str, output_path: str, data={}) -> Optional[str]:

    full_input_path = os.path.realpath(os.path.expanduser(input_path))
    input_file_dir = os.path.dirname(full_input_path)
    input_file_name = os.path.basename(full_input_path)
    environment = Environment(loader=FileSystemLoader(input_file_dir))

    template = environment.get_template(input_file_name)

    content = template.render(**data)

    full_output_path = os.path.realpath(os.path.expanduser(output_path))

    if os.path.isdir(full_output_path):
        full_output_path = f'{full_output_path}/{input_file_name}'

    with open(full_output_path, 'w', encoding='utf-8') as f:
        f.write(content)

    return full_output_path
